//
//  MenuViewController.m
//  GalerkApp
//
//  Copyright (c) 2012 Passionmakes.us. All rights reserved.
//

#import "MenuViewController.h"
#import "FlickrAPIFetcher.h"
#import "PMPhoto+Flickr.h"
#import "PMPhotoBrowser.h"

@interface MenuViewController ()

- (void)fetchFlickrData:(SEL)method;
- (void)setupAndPresentPhotoBrowser;
-(PMPhotoBrowser *)splitViewBrowserViewController;

@property (nonatomic, strong) NSArray *flickrMenu;
@property (nonatomic, strong) NSMutableArray *photosForBrowser;

@end

@implementation MenuViewController

#define ShowNetworkActivityIndicator() [UIApplication sharedApplication].networkActivityIndicatorVisible = YES
#define HideNetworkActivityIndicator() [UIApplication sharedApplication].networkActivityIndicatorVisible = NO

@synthesize photosForBrowser = _photosForBrowser, flickrMenu = _flickrMenu;

// we're loading from storyboard
- (id)initWithCoder:(NSCoder *)aDecoder
{
	if (self == [super initWithCoder:aDecoder])
	{
        // custom init
	}
	return self;
}

-(void)awakeFromNib
{
    [super awakeFromNib];
    self.splitViewController.delegate = self;
}

- (void)fetchFlickrData:(SEL)method
{
    NSMutableArray *photos = [NSMutableArray new];
    
    self.photosForBrowser = nil;
    
    dispatch_queue_t fetchQ = dispatch_queue_create("Flickr fetcher", NULL);
    
    dispatch_async(fetchQ, ^{
        
        dispatch_async(dispatch_get_main_queue(), ^{ ShowNetworkActivityIndicator(); });
        
        NSArray *fetchedFlickrInfo = [FlickrAPIFetcher performSelector:method];
        
        for (NSDictionary *flickrInfo in fetchedFlickrInfo) {
            PMPhoto *photo;
            photo = [PMPhoto photoWithFlickrInfo:flickrInfo];
            
            [photos addObject:photo];
        }
        
        self.photosForBrowser = [photos copy];
        
        dispatch_async(dispatch_get_main_queue(), ^{
          
            UILocalNotification *localNotification = [NSNotificationCenter defaultCenter];
        
            if (localNotification) {
            [[NSNotificationCenter defaultCenter] postNotificationName:@"FinishedFetchingFlickr" object:self];
            }
         });
        dispatch_async(dispatch_get_main_queue(), ^{ HideNetworkActivityIndicator(); });
        
    });
    
    dispatch_release(fetchQ);
}

-(void) prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender 
{
        
    if ([segue.identifier isEqualToString:@"Launch Photo Browser"]) {

        SEL method;
        
        NSIndexPath *selected = [self.tableView indexPathForSelectedRow];
        if ([selected row] == 0) {
            method = @selector(recentGeoreferencedPhotos);
        } else {
            method = @selector(pilinyVHlavePhotos);
        }
        
        [self fetchFlickrData:method];
//        BrowserViewController *bvc = [segue destinationViewController];
        PMPhotoBrowser *bvc = [segue destinationViewController];
        
        bvc = [bvc initWithDelegate:self];
        // hide status bar
        bvc.wantsFullScreenLayout = YES;
        [bvc reloadData];
    }
    
}

- (void) setupAndPresentPhotoBrowser
{
    // Create & present browser
    PMPhotoBrowser *browser = [[PMPhotoBrowser alloc] initWithDelegate:self];
    // options
    browser.wantsFullScreenLayout = YES; 
    
    // Present
    [self.navigationController pushViewController:browser animated:YES];
}

- (NSUInteger)numberOfPhotosInPhotoBrowser:(PMPhotoBrowser *)photoBrowser {
    return self.photosForBrowser.count;
}

- (PMPhoto *)photoBrowser:(PMPhotoBrowser *)photoBrowser photoAtIndex:(NSUInteger)index {
    if (index < self.photosForBrowser.count)
        return [self.photosForBrowser objectAtIndex:index];
    return nil;
}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section
{
    return @"Flickr";
}

- (void)viewDidLoad
{
    if (self.photosForBrowser == nil) {
        self.photosForBrowser = [[NSMutableArray alloc] init];
    }
        
    self.flickrMenu = [NSArray arrayWithObjects:NSLocalizedString(@"GEO", nil),
                          NSLocalizedString(@"PVH", nil), nil];
    
    [super viewDidLoad];

    // Uncomment the following line to preserve selection between presentations.
    // self.clearsSelectionOnViewWillAppear = NO;
 
    // Uncomment the following line to display an Edit button in the navigation bar for this view controller.
    // self.navigationItem.rightBarButtonItem = self.editButtonItem;
}

- (void)viewDidUnload
{
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
//    return (interfaceOrientation == UIInterfaceOrientationPortrait);
    return YES;
}

#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return 1;

}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    // Return the number of rows in the section.

    return self.flickrMenu.count;
    
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *CellIdentifier = @"Menu Cell";
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
    
    // Configure the cell...
    NSString *entry;
    
    if ([indexPath section] == 0) {
         entry = [self.flickrMenu objectAtIndex:indexPath.row]; 
    }
        
    cell.textLabel.text = entry;
        return cell;    
}

/*
// Override to support conditional editing of the table view.
- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Return NO if you do not want the specified item to be editable.
    return YES;
}
*/

/*
// Override to support editing the table view.
- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath
{
    if (editingStyle == UITableViewCellEditingStyleDelete) {
        // Delete the row from the data source
        [tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] withRowAnimation:UITableViewRowAnimationFade];
    }   
    else if (editingStyle == UITableViewCellEditingStyleInsert) {
        // Create a new instance of the appropriate class, insert it into the array, and add a new row to the table view
    }   
}
*/

/*
// Override to support rearranging the table view.
- (void)tableView:(UITableView *)tableView moveRowAtIndexPath:(NSIndexPath *)fromIndexPath toIndexPath:(NSIndexPath *)toIndexPath
{
}
*/

/*
// Override to support conditional rearranging of the table view.
- (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Return NO if you do not want the item to be re-orderable.
    return YES;
}
*/

#pragma mark - iPad support

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Navigation logic may go here. Create and push another view controller.
    /*
     <#DetailViewController#> *detailViewController = [[<#DetailViewController#> alloc] initWithNibName:@"<#Nib name#>" bundle:nil];
     // ...
     // Pass the selected object to the new view controller.
     [self.navigationController pushViewController:detailViewController animated:YES];
     */
    
    if ([self splitViewController]) {
    
        PMPhotoBrowser *bvc = [self splitViewBrowserViewController];
        
        SEL method;
        
        if ([indexPath row] == 0) {
            method = @selector(recentGeoreferencedPhotos);
        } else {
            method = @selector(pilinyVHlavePhotos);
        }
        
        [self fetchFlickrData:method];
        
        bvc = [bvc initWithDelegate:self];
        // hide status bar
        bvc.wantsFullScreenLayout = YES;
        [bvc reloadData];
    }
}

-(PMPhotoBrowser *)splitViewBrowserViewController
{
    id bvc = [self.splitViewController.viewControllers lastObject];
    if (![bvc isKindOfClass:[PMPhotoBrowser class]]) {
        bvc = nil;
    }
    return bvc;
}

- (BOOL)splitViewController:(UISplitViewController *)svc shouldHideViewController:     (UIViewController *)vc inOrientation:(UIInterfaceOrientation)orientation
{
    return NO;
}

@end